#include <errno.h>
#include <stdarg.h>
#include <unistd.h>

#define MAX_ARGS 100

int execl(const char *path, const char *arg, ...)
{
	va_list ap;
	char *argv[MAX_ARGS];
	int n;

	n = 0;
	argv[n] = (char *)arg;
	va_start(ap, arg);
	while (argv[n] != NULL) {
		if (n >= MAX_ARGS) {
			errno = E2BIG;
			return -1;
		}
		argv[++n] = va_arg(ap, char *);
	}
	va_end(ap);
	return execv(path, argv);
}

/* In Research UNIX sources (e.g., in V7), this is simply
 *
 * int execl(const char *path, char *arg)
 * {
 * 	return execv(path, &arg);
 * }
 *
 * Note the absence of '...' after arg in the signature.
 * The point was that the compiler translated  a call such as
 *
 * 	execl("/bin/ls", "ls", "-l", NULL);
 *
 * by pushing the arguments on the stack:
 *
 *             	+--------------+
 *  &path      	|      *-------------> "/bin/ls"
 *             	+--------------+
 *  &arg       	|      *-------------> "ls"
 *             	+--------------+
 *             	|      *-------------> "-l"
 *             	+--------------+
 *             	|    NULL      |
 *             	+--------------+
 *
 * Note how &arg points to a properly arranged argv[] vector.
 *
 * This is no longer guaranteed to work today: to access '...' arguments we
 * must use the va_list data structure and the va_*() macros, which are defined
 * by the compiler itself. This is because some arguments may be have been passed
 * through registers and only the compiler knows the details.
 */

