#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <malloc.h>

static int mem_read(FILE *f, char *c, size_t s)
{
	size_t max = s;
	if (f->pos + max < f->pos)
		return -1;
	if (f->pos + max > f->size)
		max = f->size - f->pos;
	memcpy(c, f->buf + f->pos, max);
	f->pos += max;
	return max;
}

static int mem_write(FILE *f, const char *c, size_t s)
{
	size_t max = s;
	if (f->pos + max < f->pos)
		return -1;
	if (f->pos + max > f->size)
		max = f->size - f->pos;
	memcpy(f->buf + f->pos, c, max);
	f->pos += max;
	return max;
}

static int mem_seek(FILE *f, long off, int whence)
{
	size_t absoff = off >= 0 ? off : -off;
	switch (whence) {
	case SEEK_SET:
		if (off < 0 || absoff >= f->size)
			return -1;
		f->pos = absoff;
		break;
	case SEEK_CUR:
		if ((off <  0 && absoff > f->size) ||
		    (off >= 0 && (f->pos + absoff < f->pos || f->pos + absoff > f->size)))
			return -1;
		f->pos += off;
		break;
	case SEEK_END:
		if (off > 0 || absoff >= f->size)
			return -1;
		f->pos = f->size - absoff;
		break;
	default:
		return -1;
	}
	return 0;
}

FILE *fmemopen(void *buf, size_t size, const char *mode)
{
	FILE *f;

	if ( (f = malloc(sizeof(*f))) == NULL )
		return NULL;
	f->fd = -1;
	f->buf = buf;
	f->pos = 0;
	f->size = size;
	f->read = mem_read;
	f->write = mem_write;
	f->seek = mem_seek;
	return f;
}
