#include <sys/wait.h>
#include <stdlib.h>
#include <unistd.h>

/* This is essentially the system() function as found in V7.  V7 also ignored
 * SIGINT and SIGQUIT while waiting, but we have omitted this code for
 * simplicity.
 *
 * POSIX mandates more things:
 * - we should also block SIGCHLD (a signal added later);
 * - if cmd is NULL, system() should just check that a shell is available;
 * - perhaps more importantly, we should use waitpid(pid) to wait only for the
 *   child created by system(). Our loop, instead, may throw away the return
 *   status of processes created before system() was called.
 */
 
int system(const char *cmd)
{
	int status, w;
	pid_t pid;

	switch ( (pid = fork()) ) {
	case 0:
		execl("/bin/sh", "sh", "-c", cmd, NULL);
		exit(127);
		break;
	case -1:
		return -1;
	default:
		do {
			w = wait(&status);
		} while (w != pid && w != -1);
		if (w == -1)
			status = -1;
		break;
	}
	return status;
}
