#include <sys/ioctl.h>
#include <sys/mount.h>
#include <sys/reboot.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#define MAX_LINE  1024
#define MAX_ARGS  10
#define ARGPREFIX "AL"

int main(int argc, char* argv[])
{
	int i, j;
	char buf[MAX_LINE], *v;
	char *path;
	char *args[MAX_ARGS + 1];
	char *env[MAX_ARGS + 1];
	int nargs, nenv;
	size_t plen;
	int uid = 1000;
	int gid = 1000;
	char *dir = "/";
	pid_t pid;

	plen = strlen(ARGPREFIX);
	nargs = 0;
	path = NULL;
	for (i = 1; i < argc; i++) {
		if (!strncmp(ARGPREFIX, argv[i], plen))  {
			v = argv[i] + plen;
			switch (*v) {
			case '!':
				uid = atoi(v + 1);
				break;
			case '%':
				gid = atoi(v + 1);
				break;
			case '@':
				dir = v + 1;
				break;
			case '+':
				if (nenv < MAX_ARGS) {
					for (j = 1; v[j] != '\0'; j++) {
						if (v[j] == '+') {
							v[j] = '=';
							break;
						}
					}
					env[nenv++] = v + 1;
				}
				break;
			case '&':
				path = v + 1;
				break;
			default:
				if (nargs < MAX_ARGS)
					args[nargs++] = v;
				break;
			}
		}
	}
	if (nargs == 0 && path == NULL) {
		nargs = 1;
		args[0] = "-sh";
		path = "/bin/sh";
	} else if (path != NULL) {
		nargs = 1;
		args[0] = path;
	} else {
		path = args[0];
	}
	args[nargs] = NULL;
	env[nenv] = NULL;

	mount("proc", "/proc", "proc", 0, NULL);
	system("/etc/rc");

	switch ( (pid = fork()) ) {
	case -1:
		exit(1);
	case 0:
		setsid();
		close(0);
		close(1);
		close(2);
		open("/dev/tty0", O_RDONLY);
		open("/dev/tty0", O_WRONLY);
		open("/dev/tty0", O_WRONLY);
		chown("/dev/tty0", uid, gid);
		if (strcmp(dir, "/"))
			chdir(dir);
		if (gid)
			setgid(gid);
		if (uid)
			setuid(uid);
		execve(path, args, env);
		perror(path);
		exit(1);
	default:
		while (wait(0) != pid)
			;
	}
	reboot(RB_POWER_OFF);
}
