#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <grp.h>

/* optionally change the group id too */

int main(int argc, char *argv[])
{
	uid_t u;
	gid_t g;
	int i, errors;
	struct stat statbuf;
	struct passwd *pw;
	struct group *gr;
	char *p;
	int keep_u, keep_g;

	if (argc < 3) {
		fprintf(stderr, "usage: chown [<user|uid>][:<group|gid>] <path>...\n");
		exit(1);
	}

	keep_g = 1;
	if ( (p = strchr(argv[1], ':')) ) {
		keep_g = 0;
		*p++ = '\0';
		if ( (gr = getgrnam(p)) ) {
			g = gr->gr_gid;
		} else {
			g = atoi(p);
		}
	}

	keep_u = 1;
	if (argv[1][0] != '\0') {
		keep_u = 0;
		if ( (pw = getpwnam(argv[1])) ) {
			u = pw->pw_uid;
		} else {
			u = atoi(argv[1]);
		}
	}

	errors = 0;
	for (i = 2; i < argc; i++) {
		if (keep_u || keep_g) {
			if (stat(argv[i], &statbuf) < 0) {
				perror(argv[i]);
				errors++;
				continue;
			}
			u = keep_u ? statbuf.st_uid : u;
			g = keep_g ? statbuf.st_gid : g;
		}
		if (chown(argv[i], u, g) < 0) {
			perror(argv[i]);
			errors++;
			continue;
		}
	}
	return errors;
}
