#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/wait.h>
#include <grp.h>

#define MAXTTYS 10

struct gettys {
	int pid;
	char tty[5];
} gettys[MAXTTYS];

/* spawn a new getty (on tty number i) and remember its pid */
void spawn(int i) {
	int pid;

	gettys[i].pid = 0;
	pid = fork();
	if (pid == 0) {
		/* start a new session. The tty opened by getty will become the
		 * controlling terminal for the session
		 */
		setsid();
		execl("/bin/getty", "getty", gettys[i].tty, NULL);
		exit(1);
	}
	gettys[i].pid = pid;
}

int main()
{
	int max, i, fd, pid;

	/* start a new session and reset any groups that may have
	 * been inherited from our parent (if any)
	 */
	setsid();
	setgroups(0, NULL);

	/* get the configured ttys from /etc/ttys */
	fd = open("/etc/ttys", O_RDONLY);
	max = 0;
	while(max < MAXTTYS && read(fd, gettys[max].tty, 5)) {
		gettys[max].tty[4] = '\0';
		max++;
	}
	close(fd);

	/* spawn a new getty process for each tty in /etc/ttys */
	for (i = 0; i < max; i++)
		spawn(i);

	/* Wait for child terminations. These can be either one of
	 * the getty processes that we spawned, or an orphan process.
	 * In the former case, one user has logged out and we need
	 * to respawn the getty on the same terminal. In the latter
	 * case, we do nothing (wait() has already performed the
	 * cleanup).
	 */
	for (;;) {
		pid = wait(0);

		for (i = 0; i < max; i++)
			if (gettys[i].pid == pid)
				spawn(i);
	}

	return 0;
}
