#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>

/*
 * login that creates environment variables.
 *
 * Note: the Research Unix login sets PATH, of course
 * to `:/bin:/usr/bin`.
 */

#define MAX_LINE  80

int main(int argc, char* argv[])
{
	char *pass;
	struct passwd *pw;

	/* argv[1] is the login name */
	if (argc < 2)
		return 1;

	/* getpass() turns off echo on stdout and reads the password from stdin
	 */
	pass = getpass("Password: ");

	/* getpwnam() looks up an entry in /etc/passwd */
	if ( (pw = getpwnam(argv[1])) == NULL )
		exit(1);

	/* this version of login uses unencrypted passwords */
	if (strcmp(pass, pw->pw_passwd) != 0)
		exit (1);

	/* if the password matches, we can complete the login: */

	/* - set the group first (we can no longer change it after the the
	 * setuid() if pw->pw_uid > 0)
	 */
	setgid(pw->pw_gid);
	/* - now change the uid. If pw->pw_uid is > 0, this is a permanent
	 * privileg drop. From this point on, only exec() of set-UID or set-GID
	 * programs can change the effective uid and/or gid.
	 */
	setuid(pw->pw_uid);
	/* finally chdir() to the user home directory and transform into her
	 * preferred shell
	 */
	chdir(pw->pw_dir);

	setenv("USER", pw->pw_name, 1);
	setenv("HOME", pw->pw_dir, 1);
	setenv("SHELL", pw->pw_shell, 1);

	execl(pw->pw_shell, "-sh", NULL);
	exit(1);
}
