#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#ifndef __APPLE__
#include <crypt.h>
#endif
#include <pwd.h>
#include <grp.h>

/*
 * login that sets the group of the terminal to tty
 *
 *   This enables the exchange of user messages using the
 *   write utility, which is installed as set-GID tty.
 */

#define MAX_LINE  80

int main(int argc, char* argv[])
{
	char *pass;
	struct passwd *pw;
	char salt[3];
	struct group *gr;
	gid_t ttygr;

	/* argv[1] is the login name */
	if (argc < 2)
		return 1;

	/* getpass() turns off echo on stdout and reads the password from stdin
	 */
	pass = getpass("Password: ");

	/* getpwnam() looks up an entry in /etc/passwd */
	if ( (pw = getpwnam(argv[1])) == NULL )
		exit(1);

	if (strcmp(pw->pw_passwd, "") != 0) {
		salt[0] = pw->pw_passwd[0];
		salt[1] = pw->pw_passwd[1];
		salt[2] = '\0';

		if (strcmp(crypt(pass, salt), pw->pw_passwd) != 0)
			exit (1);
	}

	/* getgrnam() looks up an entry in /etc/group */
	if ( (gr = getgrnam("tty")) != NULL )
		ttygr = gr->gr_gid;
	else
		ttygr = 0;


	/* - set the group first (we can no longer change it after the the
	 * setuid() if pw->pw_uid > 0)
	 */
	setgid(pw->pw_gid);
	/* change the terminal owner and group while we can still do it */
	fchown(0, pw->pw_uid, ttygr);
	/* - now change the uid. If pw->pw_uid is > 0, this is a permanent
	 * privileg drop. From this point on, only exec() of set-UID or set-GID
	 * programs can change the effective uid and/or gid.
	 */
	setuid(pw->pw_uid);
	/* by default, allow incoming messages using the write utility */
	fchmod(0, 0620);
	/* finally chdir() to the user home directory and transform into her
	 * preferred shell
	 */
	chdir(pw->pw_dir);

	setenv("USER", pw->pw_name, 1);
	setenv("HOME", pw->pw_dir, 1);
	setenv("SHELL", pw->pw_shell, 1);

	execl(pw->pw_shell, "-sh", NULL);
	exit(1);
}
