#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <unistd.h>

/*
 * ls with '-[ai]' options
 *
 * Each directory entry contains a mapping between names and inode numbers.
 * This is all the information that we can get by reading a directory ('r'
 * permission).
 *
 * Note: some Linux file systems also store the type of the file in the
 * directory entry.
 */

int main(int argc, char* argv[])
{
	DIR *dir;
	struct dirent* ent;
	int all = 0, inodes = 0;

	if (argc > 1)
		if (argv[1][0] == '-') {
			char* opt = &argv[1][1];
			while (*opt) {
				switch (*opt) {
				case 'a':
					all = 1;
					break;
				case 'i':
					inodes = 1;
					break;
				default:
					fprintf(stderr, "Unknown option: %c\n", *opt);
					break;
				}
				opt++;
			}
		}

	if ( !(dir = opendir(".")) ) {
		perror(".");
		return 1;
	}

	while ( (ent = readdir(dir)) ) {

		if (ent->d_name[0] == '.' && !all)
			continue;

		if (inodes)
			printf("%10llu ", (unsigned long long)ent->d_ino);

		printf("%s\n", ent->d_name);
	}

	return 0;
}
