#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <pwd.h>

#define MAX_COMM 100
#define MAXINPUT "100"
#define MAX_LINE 1000

/* decode user */
char* user(uid_t u)
{
	static char owner[11];
	struct passwd *pw;

	snprintf(owner, 11, "%d", u);

	if ( (pw = getpwuid(u)) != NULL )
		return pw->pw_name;

	return owner;
}

/* decode tty */

char *tty(unsigned int rdev)
{
	struct stat statbuf;
	static char buf[] = "/dev/tty0";
	char *p = "tp";

	for (buf[5] = *p++; buf[5] != '\0'; buf[5] = *p++) {
		for (buf[8] = '0'; buf[8] <= '9'; buf[8]++) {
			if (stat(buf, &statbuf) < 0)
				continue;
			if (statbuf.st_rdev == rdev) {
				return buf + 5;
			}
		}
	}
	return "?";
}

/* print full command */
int print_zerosep(const char *fname)
{
	FILE *f;
	int c, n;

	n = 0;
	if ( !(f = fopen("cmdline", "r")) )
		return n;
	while ( (c = fgetc(f)) != EOF) {
		n++;
		printf("%c", c == '\0' ? ' ' : c);
	}
	fclose(f);
	return n;
}

void print_command(const char *dflt)
{
	if (print_zerosep("cmdline") < 1)
		printf("[%s]", dflt);
}

void print_env()
{
	print_zerosep("environ");
}

int main(int argc, char *argv[])
{
	int pid;
	char comm[MAX_COMM + 1];
	char state;
	int ppid;
	int pgrp;
	int session;
	int tty_nr;
	int tpgid;
	DIR *dir;
	struct dirent* ent;
	FILE *f;
	char *delim;
	struct stat statbuf, statbuf2;
	int xflag = 0, aflag = 0, eflag = 0;
	uid_t uid;
	char mode = '-';

	uid = geteuid();

	if (argc > 1) {
		for (; *argv[1]; argv[1]++) {
			switch (*argv[1]) {
			case 'x':
				xflag = 1;
				break;
			case 'a':
				aflag = 1;
				break;
			case 'e':
				eflag = 1;
				break;
			case 'u':
				mode = 'u';
				break;
			case 'j':
				mode = 'j';
				break;
			}
		}
	}

	fstat(0, &statbuf);

	chdir("/proc");

	if ( !(dir = opendir(".")) ) {
		perror("/proc");
		exit(1);
	}

	switch (mode) {
	case '-':
		printf("PID\tTTY\tCMD\n");
		break;
	case 'u':
		printf("USER\tPID\tTTY\tSTAT\tCOMMAND\n");
		break;
	case 'j':
		printf("PPID\tPID\tPGID\tSID\tTTY\tTPGID\tSTAT\tUID\tCOMMAND\n");
		break;
	}
	while ( (ent = readdir(dir)) ) {
		if (ent->d_name[0] < '0' || ent->d_name[0] > '9')
			continue;
		if (chdir(ent->d_name) < 0)
			continue;
		if ( !(f = fopen("stat", "r")) ) {
			perror("stat");
			continue;
		}
		fstat(fileno(f), &statbuf2);
		if (!aflag && statbuf2.st_uid != uid)
			goto next;
		fscanf(f, "%d (%"MAXINPUT"s %c "
			  "%d %d %d %d %d ",	/* job ctrl */
			  &pid, comm, &state,
			  &ppid, &pgrp, &session, &tty_nr, &tpgid);
		comm[MAX_COMM] = '\0';
		delim = rindex(comm, ')');
		if (delim != NULL)
			*delim = '\0';
		if (!xflag && statbuf.st_rdev != (tty_nr & 0xFFFF))
			goto next;
		switch (mode) {
		case '-':
			printf("%d\t%s\t%s ", pid, tty(tty_nr), comm);
			break;
		case 'u':
			printf("%s\t%d\t%s\t%c\t", user(statbuf2.st_uid),
				pid, tty(tty_nr), state);
			print_command(comm);
			break;
		case 'j':
			printf("%d\t%d\t%d\t%d\t%s\t%d\t%c\t%u\t",
					ppid, pid, pgrp, session, tty(tty_nr),
					tpgid, state, statbuf2.st_uid);
			print_command(comm);
			break;
		}
		if (eflag)
			print_env();
		printf("\n");
	next:
		fclose(f);
		chdir("..");
	}

	return 0;
}
