#include <sys/wait.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

/* We only know the inode number of the current directory.
 * From that, we have to discover the symbolic path by ourselves.
 *
 * Note: we should also check when we cross mount points, but we don't,
 * for simplicity.
 */

#define BUFSZ 256

int dot_inode()
{
	DIR *dir;
	struct dirent* ent;

	if (! (dir = opendir(".")) )
		return 0;
	while ( (ent = readdir(dir)) )
		if (strcmp(ent->d_name, ".") == 0)
			break;
	closedir(dir);
	if (ent == NULL)
		return 0; // can't happen
	return ent->d_ino;
}

void print_path(ino_t i, int depth)
{
	DIR *dir;
	struct dirent *tmp;
	char buf[BUFSZ];
	ino_t j;

	if (chdir("..") < 0)
		return;
	j = dot_inode();
	if (j == i) {
		if (!depth)
			printf("/");
		return;
	}

	if ( !(dir = opendir(".")) )
		return;
	while ( (tmp = readdir(dir)) ) {
		if (tmp->d_ino == i) {
			strncpy(buf, tmp->d_name, BUFSZ);
			buf[BUFSZ-1] = '\0';
			closedir(dir);
			print_path(j, depth + 1);
			printf("/%s", buf);
			break;
		}
	}
}

int main()
{
	ino_t i = dot_inode();

	print_path(i, 0);
	printf("\n");

	return 0;
}
