#include <sys/stat.h>
#include <ctype.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>

int find_tty(uid_t uid, char *buf, struct stat *statbuf)
{
	char *p;

	for (p = "tp", buf[5] = *p++; buf[5] != '\0'; buf[5] = *p++) {
		for (buf[8] = '0'; buf[8] <= '9'; buf[8]++) {
			if (stat(buf, statbuf) < 0)
				continue;
			if (statbuf->st_uid == uid) {
				return 1;
			}
		}
	}
	return 0;
}

int main(int argc, char *argv[])
{
	struct passwd *pw1, *pw2;
	struct stat statbuf1, statbuf2;
	char buf1[] = "/dev/tty0";
	char buf2[] = "/dev/tty0";
	unsigned char c;

	if (argc != 2) {
		fprintf(stderr, "usage: write <user>\n");
		return 1;
	}

	if ( (pw1 = getpwuid(getuid())) == NULL ) {
		fprintf(stderr, "internal error\n");
		return 1;
	}

	if ( (pw2 = getpwnam(argv[1])) == NULL ) {
		fprintf(stderr, "no such user\n");
		return 1;
	}

	if (getuid() && (!find_tty(pw1->pw_uid, buf1, &statbuf1) || !(statbuf1.st_mode & 0020))) {
		fprintf(stderr, "cannot receive replies\n");
		return 1;
	}

	if (!find_tty(pw2->pw_uid, buf2, &statbuf2)) {
		fprintf(stderr, "not logged in\n");
		return 1;
	}
	if (getuid() && !(statbuf2.st_mode & 0020)) {
		fprintf(stderr, "not accepting messages\n");
		return 1;
	}

	close(1);
	if (open(buf2, O_WRONLY) < 0) {
		perror(buf2);
		return 1;
	}
	printf("Message from %s on %s\n", pw1->pw_name, buf1);

	while (read(0, &c, 1) > 0) {
		if (c < 32 && !isspace(c)) {
			char buf[2] = "^";
			buf[1] = c ^ 0x40;
			write(1, buf, 2);
		} else if (c > 127) {
			char buf[4] = "\\";
			buf[1] = '0' + (c / 64);
			buf[2] = '0' + ((c % 64) / 8);
			buf[3] = '0' + (c % 8);
			write(1, buf, 4);
		} else {
			write(1, &c, 1);
		}
	}
	write(1, "EOF\n", 4);
}
